@@ -27,13 +27,13 @@ Copyright 1987 by Digital Equipment Corporation, Maynard, Massachusetts.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the name of Digital not be
 used in advertising or publicity pertaining to distribution of the
-software without specific, written prior permission.  
+software without specific, written prior permission.
 
 DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
@@ -53,19 +53,19 @@ SOFTWARE.
  fee is hereby granted, provided that the above copyright
  notice appear in all copies and that both that copyright
  notice and this permission notice appear in supporting
- documentation, and that the name of Silicon Graphics not be 
- used in advertising or publicity pertaining to distribution 
+ documentation, and that the name of Silicon Graphics not be
+ used in advertising or publicity pertaining to distribution
  of the software without specific prior written permission.
- Silicon Graphics makes no representation about the suitability 
+ Silicon Graphics makes no representation about the suitability
  of this software for any purpose. It is provided "as is"
  without any express or implied warranty.
- 
- SILICON GRAPHICS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS 
- SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY 
+
+ SILICON GRAPHICS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
+ SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL SILICON
- GRAPHICS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
- DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, 
- DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE 
+ GRAPHICS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+ DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
+ DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
  OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
  THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
@@ -89,95 +89,100 @@ SOFTWARE.
 #define InitialTableSize 100
 
 typedef struct _Node {
-    struct _Node   *left,   *right;
+    struct _Node *left, *right;
     Atom a;
     unsigned int fingerPrint;
-    char   *string;
+    char *string;
 } NodeRec, *NodePtr;
 
 #define BAD_RESOURCE 0xe0000000
 
 static Atom lastAtom = None;
-static NodePtr atomRoot = (NodePtr)NULL;
+static NodePtr atomRoot = (NodePtr) NULL;
 static unsigned long tableLength;
-static NodePtr *nodeTable;
+static NodePtr *nodeTable = NULL;
 
-static Atom 
-_XkbMakeAtom(char *string,unsigned len,Bool makeit)
+static Atom
+_XkbMakeAtom(const char *string, size_t len, Bool makeit)
 {
-    register    NodePtr * np;
+    register NodePtr *np;
     unsigned i;
-    int     comp;
-    register unsigned int   fp = 0;
+    int comp;
+    register unsigned int fp = 0;
 
     np = &atomRoot;
-    for (i = 0; i < (len+1)/2; i++)
-    {
-	fp = fp * 27 + string[i];
-	fp = fp * 27 + string[len - 1 - i];
+    for (i = 0; i < (len + 1) / 2; i++) {
+        fp = fp * 27 + string[i];
+        fp = fp * 27 + string[len - 1 - i];
     }
-    while (*np != (NodePtr) NULL)
-    {
-	if (fp < (*np)->fingerPrint)
-	    np = &((*np)->left);
-	else if (fp > (*np)->fingerPrint)
-	    np = &((*np)->right);
-	else
-	{			       /* now start testing the strings */
-	    comp = strncmp(string, (*np)->string, (int)len);
-	    if ((comp < 0) || ((comp == 0) && (len < strlen((*np)->string))))
-		np = &((*np)->left);
-	    else if (comp > 0)
-		np = &((*np)->right);
-	    else
-		return(*np)->a;
-	    }
+    while (*np != (NodePtr) NULL) {
+        if (fp < (*np)->fingerPrint)
+            np = &((*np)->left);
+        else if (fp > (*np)->fingerPrint)
+            np = &((*np)->right);
+        else {                  /* now start testing the strings */
+            comp = strncmp(string, (*np)->string, len);
+            if ((comp < 0) || ((comp == 0) && (len < strlen((*np)->string))))
+                np = &((*np)->left);
+            else if (comp > 0)
+                np = &((*np)->right);
+            else
+                return (*np)->a;
+        }
     }
-    if (makeit)
-    {
-	register NodePtr nd;
+    if (makeit) {
+        register NodePtr nd;
 
-	nd = (NodePtr) _XkbAlloc(sizeof(NodeRec));
-	if (!nd)
-	    return BAD_RESOURCE;
-	nd->string = (char *) _XkbAlloc(len + 1);
-	if (!nd->string) {
-	    _XkbFree(nd);
-	    return BAD_RESOURCE;
-	}
-	strncpy(nd->string, string, (int)len);
-	nd->string[len] = 0;
-	if ((lastAtom + 1) >= tableLength) {
-	    NodePtr *table;
+        nd = (NodePtr) _XkbAlloc(sizeof(NodeRec));
+        if (!nd)
+            return BAD_RESOURCE;
+#ifdef HAVE_STRNDUP
+        nd->string = strndup(string, len);
+#else
+        nd->string = (char *) _XkbAlloc(len + 1);
+#endif
+        if (!nd->string) {
+            _XkbFree(nd);
+            return BAD_RESOURCE;
+        }
+#ifndef HAVE_STRNDUP
+        strncpy(nd->string, string, len);
+        nd->string[len] = 0;
+#endif
+        if ((lastAtom + 1) >= tableLength) {
+            NodePtr *table;
 
-	    table = (NodePtr *) _XkbRealloc(nodeTable,
-					 tableLength * (2 * sizeof(NodePtr)));
-	    if (!table) {
-		if (nd->string != string)
-		    _XkbFree(nd->string);
-		_XkbFree(nd);
-		return BAD_RESOURCE;
-	    }
-	    tableLength <<= 1;
-	    nodeTable = table;
-	}
-	*np = nd;
-	nd->left = nd->right = (NodePtr) NULL;
-	nd->fingerPrint = fp;
-	nd->a = (++lastAtom);
-	*(nodeTable+lastAtom) = nd;
-	return nd->a;
+            table = (NodePtr *) _XkbRealloc(nodeTable,
+                                            tableLength * (2 *
+                                                           sizeof(NodePtr)));
+            if (!table) {
+                _XkbFree(nd->string);
+                _XkbFree(nd);
+                return BAD_RESOURCE;
+            }
+            tableLength <<= 1;
+            nodeTable = table;
+        }
+        *np = nd;
+        nd->left = nd->right = (NodePtr) NULL;
+        nd->fingerPrint = fp;
+        nd->a = (++lastAtom);
+        *(nodeTable + lastAtom) = nd;
+        return nd->a;
     }
     else
-	return None;
+        return None;
 }
 
 static char *
 _XkbNameForAtom(Atom atom)
 {
     NodePtr node;
-    if (atom > lastAtom) return NULL;
-    if ((node = nodeTable[atom]) == (NodePtr)NULL) return NULL;
+
+    if (atom > lastAtom)
+        return NULL;
+    if ((node = nodeTable[atom]) == (NodePtr) NULL)
+        return NULL;
     return strdup(node->string);
 }
 
@@ -185,46 +190,48 @@ static void
 _XkbInitAtoms(void)
 {
     tableLength = InitialTableSize;
-    nodeTable = (NodePtr *)_XkbAlloc(InitialTableSize*sizeof(NodePtr));
-    nodeTable[None] = (NodePtr)NULL;
+    nodeTable = (NodePtr *) _XkbAlloc(InitialTableSize * sizeof(NodePtr));
+    if (nodeTable != NULL)
+        nodeTable[None] = (NodePtr) NULL;
 }
-    
+
 /***====================================================================***/
 
 char *
-XkbAtomGetString(Display *dpy,Atom atm)
+XkbAtomGetString(Display *dpy, Atom atm)
 {
-    if (atm==None)
-	return NULL;
-    if (dpy==NULL)
-	return _XkbNameForAtom(atm);
-    return XGetAtomName(dpy,atm);
+    if (atm == None)
+        return NULL;
+    if (dpy == NULL)
+        return _XkbNameForAtom(atm);
+    return XGetAtomName(dpy, atm);
 }
 
 /***====================================================================***/
 
 Atom
-XkbInternAtom(Display *dpy,char *name,Bool onlyIfExists)
+XkbInternAtom(Display *dpy, const char *name, Bool onlyIfExists)
 {
-    if (name==NULL)
-	return None;
-    if (dpy==NULL) {
-	return _XkbMakeAtom(name, strlen(name), (!onlyIfExists));
+    if (name == NULL)
+        return None;
+    if (dpy == NULL) {
+        return _XkbMakeAtom(name, strlen(name), (!onlyIfExists));
     }
-    return XInternAtom(dpy,name,onlyIfExists);
+    return XInternAtom(dpy, name, onlyIfExists);
 }
 
 /***====================================================================***/
 
 Atom
-XkbChangeAtomDisplay(Display *oldDpy,Display *newDpy,Atom atm)
+XkbChangeAtomDisplay(Display *oldDpy, Display *newDpy, Atom atm)
 {
-char *tmp;
-
-    if (atm!=None) {
-	tmp= XkbAtomGetString(oldDpy,atm);
-	if (tmp!=NULL)
-	    return XkbInternAtom(newDpy,tmp,False);
+    if (atm != None) {
+        char *tmp = XkbAtomGetString(oldDpy, atm);
+        if (tmp != NULL) {
+            Atom a = XkbInternAtom(newDpy, tmp, False);
+            _XkbFree(tmp);
+            return a;
+        }
     }
     return None;
 }
@@ -234,10 +241,8 @@ char *tmp;
 void
 XkbInitAtoms(Display *dpy)
 {
-static int been_here= 0;
-    if ((dpy==NULL)&&(!been_here)) {
-	_XkbInitAtoms();
-	been_here= 1;
+    if ((dpy == NULL) && (nodeTable == NULL)) {
+        _XkbInitAtoms();
     }
     return;
 }
